@@ -1,9 +1,11 @@
 import re
-from copy import deepcopy
-
-import pandas as pd
 import datetime
+from copy import deepcopy
 from dateutil.parser import parse as parse_dt
+from typing import Dict, List, Optional, Tuple, Callable, Union
+
+import numpy as np
+import pandas as pd
 
 from lightwood.api.dtype import dtype
 from lightwood.helpers import text
@@ -11,8 +13,6 @@ from lightwood.helpers.log import log
 from lightwood.api.types import TimeseriesSettings
 from lightwood.helpers.numeric import is_nan_numeric
 
-from typing import Dict, List, Optional, Tuple, Callable, Union
-
 
 def cleaner(
     data: pd.DataFrame,
@@ -43,6 +43,8 @@ def cleaner(
     data = _remove_columns(data, identifiers, target, mode, timeseries_settings,
                            anomaly_detection, dtype_dict)
 
+    data['__mdb_original_index'] = np.arange(len(data))
+
     for col in _get_columns_to_clean(data, dtype_dict, mode, target):
 
         # Get and apply a cleaning function for each data type
